@@ -157,3 +157,44 @@ class Get(ClauseOperator):
     @classmethod
     def generate_origin_expression(cls, operand, component) -> str:
         return f"{operand.name} [ get {component} ]"
+
+
+class Sub(ClauseOperator):
+    op = tokens.SUB
+    propagate_attributes = True
+
+    @classmethod
+    def validate(cls, operand, property_code, value):
+        if not isinstance(operand, RecordSet):
+            raise exceptions.SemanticError("4-5-0-2", operator=cls.op)
+
+        # Validate that the property_code exists in the operand's components
+        dpm_components = operand.get_dpm_components()
+        if property_code not in dpm_components:
+            raise exceptions.SemanticError("2-8", op=cls.op, dpm_keys=[property_code], recordset=operand.name)
+
+        # Generate origin expression
+        origin = cls.generate_origin_expression(operand, property_code, value)
+
+        # Generate a new label for the result
+        new_label = generate_new_label()
+        operand.structure.replace_components_parent(new_label)
+
+        # The sub operator drops the specified component from the structure
+        if property_code in operand.structure.components:
+            del operand.structure.components[property_code]
+
+        # Remove attributes if not propagating
+        if not cls.propagate_attributes:
+            operand.structure.remove_attributes()
+
+        result = RecordSet(structure=operand.structure, name=new_label, origin=origin)
+        result.records = operand.records
+        set_operand_label(result.name, result.origin)
+        return result
+
+    @classmethod
+    def generate_origin_expression(cls, operand, property_code, value) -> str:
+        operand_name = getattr(operand, 'name', None) or getattr(operand, 'origin', None)
+        value_str = getattr(value, 'name', None) or getattr(value, 'origin', None) or str(value)
+        return f"{operand_name}[sub {property_code} = {value_str}]"
