@@ -2,6 +2,16 @@ from py_dpm.api.migration import MigrationAPI
 from py_dpm.api.syntax import SyntaxAPI
 from py_dpm.api.semantic import SemanticAPI
 from py_dpm.api.data_dictionary import DataDictionaryAPI
+from py_dpm.api.operation_scopes import (
+    OperationScopesAPI,
+    calculate_scopes_from_expression,
+    get_existing_scopes,
+    ModuleVersionInfo,
+    TableVersionInfo,
+    HeaderVersionInfo,
+    OperationScopeDetailedInfo,
+    OperationScopeResult
+)
 from py_dpm.api.ast_generator import ASTGenerator, parse_expression, validate_expression, parse_batch
 from py_dpm.api.complete_ast import (
     generate_complete_ast,
@@ -55,6 +65,19 @@ __all__ = [
     'SyntaxAPI',
     'SemanticAPI',
     'DataDictionaryAPI',
+    'OperationScopesAPI',
+
+    # Operation Scopes Convenience Functions
+    'calculate_scopes_from_expression',
+    'get_existing_scopes',
+
+    # Operation Scopes Data Classes
+    'ModuleVersionInfo',
+    'TableVersionInfo',
+    'HeaderVersionInfo',
+    'OperationScopeDetailedInfo',
+    'OperationScopeResult',
+
     'API'  # Keep for backward compatibility
 ]
 
@@ -73,12 +96,11 @@ class API:
         """
         if connection_url:
             # Create isolated engine and session for the provided connection URL
-            from sqlalchemy import create_engine
             from sqlalchemy.orm import sessionmaker
+            from py_dpm.db_utils import create_engine_from_url
 
-            # Create engine for the connection URL (PostgreSQL, MySQL, etc.)
-            self.engine = create_engine(connection_url, pool_pre_ping=True,
-                                       pool_size=20, max_overflow=10, pool_recycle=180)
+            # Create engine for the connection URL (supports SQLite, PostgreSQL, MySQL, etc.)
+            self.engine = create_engine_from_url(connection_url)
             session_maker = sessionmaker(bind=self.engine)
             self.session = session_maker()
         elif database_path:
